@@ -258,7 +258,7 @@ class BackendCardDAV extends BackendDiff implements ISearchProvider {
 
             if ($vcards === false) {
                 ZLog::Write(LOGLEVEL_ERROR, sprintf("BackendCardDAV->ChangesSink - Error getting the changes"));
-                return false;
+                continue;
             }
             else {
                 $xml_vcards = new SimpleXMLElement($vcards);
@@ -747,13 +747,15 @@ class BackendCardDAV extends BackendDiff implements ISearchProvider {
     /**
      * Queries the CardDAV backend
      *
-     * @param string        $searchquery        string to be searched for
-     * @param string        $searchrange        specified searchrange
+     * @param string                        $searchquery        string to be searched for
+     * @param string                        $searchrange        specified searchrange
+     * @param SyncResolveRecipientsPicture  $searchpicture      limitations for picture
      *
      * @access public
      * @return array        search results
+     * @throws StatusException
      */
-    public function GetGALSearchResults($searchquery, $searchrange) {
+    public function GetGALSearchResults($searchquery, $searchrange, $searchpicture) {
         ZLog::Write(LOGLEVEL_DEBUG, sprintf("BackendCardDAV->GetGALSearchResults(%s, %s)", $searchquery, $searchrange));
         if ($this->gal_url !== false && $this->server !== false) {
             // Don't search if the length is < 5, we are typing yet
@@ -795,7 +797,7 @@ class BackendCardDAV extends BackendDiff implements ISearchProvider {
             // TODO the limiting of the searchresults could be refactored into Utils as it's probably used more than once
             $querycnt = $xml_vcards->count();
             //do not return more results as requested in range
-            $querylimit = (($rangeend + 1) < $querycnt) ? ($rangeend + 1) : $querycnt == 0 ? 1 : $querycnt;
+            $querylimit = (($rangeend + 1) < $querycnt) ? ($rangeend + 1) : ($querycnt == 0 ? 1 : $querycnt);
             $items['range'] = $rangestart.'-'.($querylimit - 1);
             $items['searchtotal'] = $querycnt;
 
@@ -1079,6 +1081,15 @@ class BackendCardDAV extends BackendDiff implements ISearchProvider {
                 elseif (in_array('cell', $tel['type'])) {
                     $message->mobilephonenumber = $tel['val'][0];
                 }
+                elseif (in_array('main', $tel['type'])) {
+                    $message->companymainphone = $tel['val'][0];
+                }
+                elseif (in_array('assistant', $tel['type'])) {
+                    $message->assistnamephonenumber = $tel['val'][0];
+                }
+                elseif (in_array('text', $tel['type'])) {
+                    $message->mms = $tel['val'][0];
+                }
                 elseif (in_array('home', $tel['type'])) {
                     if (in_array('fax', $tel['type'])) {
                         $message->homefaxnumber = $tel['val'][0];
@@ -1240,8 +1251,12 @@ class BackendCardDAV extends BackendDiff implements ISearchProvider {
             'home2phonenumber' => 'TEL;TYPE=HOME,VOICE',
             'homefaxnumber' => 'TEL;TYPE=HOME,FAX',
             'mobilephonenumber' => 'TEL;TYPE=CELL',
-            'carphonenumber' => 'TEL;TYPE=VOICE',
+            'carphonenumber' => 'TEL;TYPE=CAR',
             'pagernumber' => 'TEL;TYPE=PAGER',
+            'companymainphone' => 'TEL;TYPE=WORK,MAIN',
+            'mms' => 'TEL;TYPE=TEXT',
+            'radiophonenumber' => 'TEL;TYPE=RADIO,VOICE',
+            'assistnamephonenumber' => 'TEL;TYPE=ASSISTANT,VOICE',
             ';;businessstreet;businesscity;businessstate;businesspostalcode;businesscountry' => 'ADR;TYPE=WORK',
             ';;homestreet;homecity;homestate;homepostalcode;homecountry' => 'ADR;TYPE=HOME',
             ';;otherstreet;othercity;otherstate;otherpostalcode;othercountry' => 'ADR',
@@ -1261,7 +1276,7 @@ class BackendCardDAV extends BackendDiff implements ISearchProvider {
                     $val .= $this->escape($message->$i);
                 $val.=';';
             }
-            if ($k == 'body' && isset($message->asbody)) {
+            if ($k == 'body' && isset($message->asbody->data)) {
                 $val = stream_get_contents($message->asbody->data);
             }
             if (empty($val) || preg_match('/^(\;)+$/', $val) == 1)
@@ -1270,7 +1285,11 @@ class BackendCardDAV extends BackendDiff implements ISearchProvider {
             $val = str_replace("\n", "\\n", $val);
 
             // Remove trailing ;
-            $val = substr($val, 0, -1);
+            $val = rtrim($val, ";");
+            // Clean full name from emailaddress
+            if (substr($k, 0, 5) == 'email') {
+                $val = preg_replace(array('/.*</', '/>.*/'), array('', ''), $val);
+            }
             if (strlen($val) > 50) {
                 $data .= $v.":\n\t".substr(chunk_split($val, 50, "\n\t"), 0, -1);
             }
@@ -1288,7 +1307,7 @@ class BackendCardDAV extends BackendDiff implements ISearchProvider {
 
         // http://en.wikipedia.org/wiki/VCard
         // TODO: add support for v4.0
-        // not supported: anniversary, assistantname, assistnamephonenumber, children, department, officelocation, radiophonenumber, spouse, rtf
+        // not supported: anniversary, assistantname, children, department, officelocation, spouse, rtf
 
         return $data;
     }
