@@ -299,12 +299,12 @@
 
             // Update body of original message
             $msgbody = mapi_openproperty($this->message, PR_BODY);
-            $msgbody = trim($this->windows1252_to_utf8($msgbody), "\0");
+            $msgbody = trim($msgbody, "\0");
             $separator = "------------\r\n";
 
             if (!empty($msgbody) && strrpos($msgbody, $separator) === false) {
                 $msgbody = $separator . $msgbody;
-                $stream = mapi_openproperty($this->message, PR_BODY, IID_IStream, 0, MAPI_CREATE | MAPI_MODIFY);
+                $stream = mapi_openproperty($this->message, PR_BODY, IID_IStream, STGM_TRANSACTED, 0);
                 mapi_stream_setsize($stream, strlen($msgbody));
                 mapi_stream_write($stream, $msgbody);
                 mapi_stream_commit($stream);
@@ -329,7 +329,7 @@
                 $newItem[$this->proptags['startdate']] = $now;
 
                 // If startdate and enddate are set on task, then slide enddate according to duration
-                if (isset($this->messageprops[$this->proptags["startdate"]]) && isset($this->messageprops[$this->proptags["duedate"]])) {
+                if (isset($this->messageprops[$this->proptags["startdate"]], $this->messageprops[$this->proptags["duedate"]])) {
                     $newItem[$this->proptags['duedate']] = $newItem[$this->proptags['startdate']] + ($this->messageprops[$this->proptags["duedate"]] - $this->messageprops[$this->proptags["startdate"]]);
                 } else {
                     $newItem[$this->proptags['duedate']] = $newItem[$this->proptags['startdate']];
@@ -409,21 +409,4 @@
 
             return $result;
         }
-
-        /**
-        * Convert from windows-1252 encoded string to UTF-8 string
-        *
-        * The same conversion rules as utf8_to_windows1252 apply.
-        *
-        * @param string $string the Windows-1252 string to convert
-        * @return string UTF-8 representation of the string
-        */
-        function windows1252_to_utf8($string)
-        {
-            if (function_exists("iconv")){
-                return iconv("Windows-1252", "UTF-8//TRANSLIT", $string);
-            }else{
-                return utf8_encode($string); // no euro support here
-            }
-        }
     }
