@@ -393,11 +393,11 @@ class MAPIUtils {
         $stream = mapi_openproperty($message, $prop, IID_IStream, 0, 0);
         $ret = mapi_last_hresult();
         if ($ret == MAPI_E_NOT_FOUND) {
-            ZLog::Write(LOGLEVEL_DEBUG, sprintf("MAPIUtils->readPropStream: property 0x%s not found. It is either empty or not set. It will be ignored.", str_pad(dechex($prop), 8, 0, STR_PAD_LEFT)));
+            ZLog::Write(LOGLEVEL_DEBUG, sprintf("MAPIUtils->readPropStream: property 0x%08X not found. It is either empty or not set. It will be ignored.", $prop));
             return "";
         }
         elseif ($ret) {
-            ZLog::Write(LOGLEVEL_ERROR, "MAPIUtils->readPropStream error opening stream: 0X%X", $ret);
+            ZLog::Write(LOGLEVEL_ERROR, "MAPIUtils->readPropStream error opening stream: 0x%08X", $ret);
             return "";
         }
         $data = "";
@@ -728,7 +728,7 @@ class MAPIUtils {
 
         $res['server'] = substr($entryId, $offset);
 
-        $res['min_lenth'] = 64;
+        $res['min_length'] = 64;
         $res['name'] = 'EID_V0';
 
         return $res;
