@@ -538,7 +538,7 @@ class Mail_mimeDecode
 
                 $hdr_name = substr($value, 0, $pos = strpos($value, ':'));
                 $hdr_value = substr($value, $pos+1);
-                if($hdr_value[0] == ' ') {
+                if(strlen($hdr_value) && $hdr_value[0] == ' ') {
                     $hdr_value = substr($hdr_value, 1);
                 }
 
@@ -883,10 +883,9 @@ class Mail_mimeDecode
         $input = preg_replace("/=\r?\n/", '', $input);
 
         // Replace encoded characters
-
-        $cb = create_function('$matches',  ' return chr(hexdec($matches[0]));');
-
-        $input = preg_replace_callback( '/=([a-f0-9]{2})/i', $cb, $input);
+        $input = preg_replace_callback( '/=([a-f0-9]{2})/i',
+                                        function ($match) {return chr(hexdec($match[0]));},
+                                        $input);
 
         return $input;
     }
