@@ -7,29 +7,11 @@
 *
 * Created   :   11.04.2011
 *
-* Copyright 2007 - 2013 Zarafa Deutschland GmbH
+* Copyright 2007 - 2016 Zarafa Deutschland GmbH
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3,
-* as published by the Free Software Foundation with the following additional
-* term according to sec. 7:
-*
-* According to sec. 7 of the GNU Affero General Public License, version 3,
-* the terms of the AGPL are supplemented with the following terms:
-*
-* "Zarafa" is a registered trademark of Zarafa B.V.
-* "Z-Push" is a registered trademark of Zarafa Deutschland GmbH
-* The licensing of the Program under the AGPL does not imply a trademark license.
-* Therefore any rights, title and interest in our trademarks remain entirely with us.
-*
-* However, if you propagate an unmodified version of the Program you are
-* allowed to use the term "Z-Push" to indicate that you distribute the Program.
-* Furthermore you may use our trademarks where it is necessary to indicate
-* the intended purpose of a product or service provided you use it in accordance
-* with honest practices in industrial or commercial matters.
-* If you want to propagate modified versions of the Program under the name "Z-Push",
-* you may only do so if you have a written permission by Zarafa Deutschland GmbH
-* (to acquire a permission please contact Zarafa at trademark@zarafa.com).
+* as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -73,13 +55,19 @@ class ContentParameters extends StateObject {
 
     /**
      * Overwrite StateObject->__call so we are able to handle ContentParameters->BodyPreference()
+     * and ContentParameters->BodyPartPreference().
      *
      * @access public
      * @return mixed
      */
     public function __call($name, $arguments) {
-        if ($name === "BodyPreference")
+        if ($name === "BodyPreference") {
             return $this->BodyPreference($arguments[0]);
+        }
+
+        if ($name === "BodyPartPreference") {
+            return $this->BodyPartPreference($arguments[0]);
+        }
 
         return parent::__call($name, $arguments);
     }
@@ -108,6 +96,30 @@ class ContentParameters extends StateObject {
         }
     }
 
+    /**
+     * Instantiates/returns the bodypartpreference object for a type.
+     *
+     * @param int   $type
+     *
+     * @access public
+     * @return int/boolean          returns false if value is not defined
+     */
+    public function BodyPartPreference($type) {
+        if (!isset($this->bodypartpref)) {
+            $this->bodypartpref = array();
+        }
+
+        if (isset($this->bodypartpref[$type])) {
+            return $this->bodypartpref[$type];
+        }
+
+        $asb = new BodyPartPreference();
+        $arr = (array)$this->bodypartpref;
+        $arr[$type] = $asb;
+        $this->bodypartpref = $arr;
+        return $asb;
+    }
+
     /**
      * Returns available body preference objects
      *
@@ -122,6 +134,20 @@ class ContentParameters extends StateObject {
         return array_keys($this->bodypref);
     }
 
+    /**
+     * Returns available body part preference objects.
+     *
+     *  @access public
+     *  @return array/boolean       returns false if the client's body preference is not available
+     */
+    public function GetBodyPartPreference() {
+        if (!isset($this->bodypartpref) || !(is_array($this->bodypartpref) || empty($this->bodypartpref))) {
+            ZLog::Write(LOGLEVEL_DEBUG, sprintf("ContentParameters->GetBodyPartPreference(): bodypartpref is empty or not set"));
+            return false;
+        }
+        return array_keys($this->bodypartpref);
+    }
+
     /**
      * Called before the StateObject is serialized
      *
