@@ -11,29 +11,11 @@
 *
 * Created   :   20.10.2011
 *
-* Copyright 2007 - 2015 Zarafa Deutschland GmbH
+* Copyright 2007 - 2016 Zarafa Deutschland GmbH
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3,
-* as published by the Free Software Foundation with the following additional
-* term according to sec. 7:
-*
-* According to sec. 7 of the GNU Affero General Public License, version 3,
-* the terms of the AGPL are supplemented with the following terms:
-*
-* "Zarafa" is a registered trademark of Zarafa B.V.
-* "Z-Push" is a registered trademark of Zarafa Deutschland GmbH
-* The licensing of the Program under the AGPL does not imply a trademark license.
-* Therefore any rights, title and interest in our trademarks remain entirely with us.
-*
-* However, if you propagate an unmodified version of the Program you are
-* allowed to use the term "Z-Push" to indicate that you distribute the Program.
-* Furthermore you may use our trademarks where it is necessary to indicate
-* the intended purpose of a product or service provided you use it in accordance
-* with honest practices in industrial or commercial matters.
-* If you want to propagate modified versions of the Program under the name "Z-Push",
-* you may only do so if you have a written permission by Zarafa Deutschland GmbH
-* (to acquire a permission please contact Zarafa at trademark@zarafa.com).
+* as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -46,7 +28,6 @@
 * Consult LICENSE file for details
 ************************************************/
 
-
 class LoopDetection extends InterProcessData {
     const INTERPROCESSLD = "ipldkey";
     const BROKENMSGS = "bromsgs";
@@ -62,7 +43,7 @@ class LoopDetection extends InterProcessData {
      *
      * @access public
      */
-    public function LoopDetection() {
+    public function __construct() {
         // initialize super parameters
         $this->allocate = 1024000; // 1 MB
         $this->type = 1337;
@@ -162,6 +143,7 @@ class LoopDetection extends InterProcessData {
      * @return boolean
      */
     public function ProcessLoopDetectionAddStatus($folderid, $status) {
+        ZLog::Write(LOGLEVEL_DEBUG, sprintf("LoopDetection->ProcessLoopDetectionAddStatus: '%s' with status %d", $folderid?$folderid:'hierarchy', $status));
         // generate entry if not already there
         self::GetProcessEntry();
 
@@ -192,6 +174,73 @@ class LoopDetection extends InterProcessData {
         return $this->updateProcessStack();
     }
 
+    /**
+     * Indicates if a simple Hierarchy sync should be done after Ping.
+     *
+     * When trying to sync a non existing folder, Sync will return Status 12.
+     * This should trigger a hierarchy sync by the client, but this is not always done.
+     * Clients continue trying to Ping, which fails as well and triggers a Sync again.
+     * This goes on forever, like here: https://jira.z-hub.io/browse/ZP-1077
+     *
+     * Ping could indicate to perform a FolderSync as well after a few Sync/Ping cycles.
+     *
+     * @access public
+     * @return boolean
+     *
+     */
+    public function ProcessLoopDetectionIsHierarchySyncAdvised() {
+        $me = self::GetProcessEntry();
+        if ($me['cc'] !== ZPush::COMMAND_PING) {
+            return false;
+        }
+
+        $loopingFolders = array();
+
+        $lookback = self::$start - 600; // look at the last 5 min
+        foreach ($this->getProcessStack() as $se) {
+            if ($se['time'] > $lookback && $se['time'] < (self::$start-1)) {
+                // look for sync command
+                if (isset($se['stat']) && ($se['cc'] == ZPush::COMMAND_SYNC || $se['cc'] == ZPush::COMMAND_PING)) {
+                    foreach($se['stat'] as $key => $value) {
+                        // we only care about hierarchy errors of this folder
+                        if ($se['cc'] == ZPush::COMMAND_SYNC) {
+                            if ($value == SYNC_STATUS_FOLDERHIERARCHYCHANGED) {
+                                ZLog::Write(LOGLEVEL_DEBUG, sprintf("LoopDetection->ProcessLoopDetectionIsHierarchySyncAdvised(): seen Sync command with Exception or folderid '%s' and code '%s'", $key, $value ));
+                            }
+                        }
+                        if (!isset($loopingFolders[$key])) {
+                            $loopingFolders[$key] = array(ZPush::COMMAND_SYNC => array(), ZPush::COMMAND_PING => array());
+                        }
+                        if (!isset($loopingFolders[$key][$se['cc']][$value])) {
+                            $loopingFolders[$key][$se['cc']][$value] = 0;
+                        }
+                        $loopingFolders[$key][$se['cc']][$value]++;
+                    }
+                }
+            }
+        }
+
+        $filtered = array();
+        foreach ($loopingFolders as $fid => $data) {
+            // Ping is constantly generating changes for this folder
+            if (isset($data[ZPush::COMMAND_PING][SYNC_PINGSTATUS_CHANGES]) && $data[ZPush::COMMAND_PING][SYNC_PINGSTATUS_CHANGES] >= 3) {
+                // but the Sync request is not treating it (not being requested)
+                if (count($data[ZPush::COMMAND_SYNC]) == 0) {
+                    ZLog::Write(LOGLEVEL_INFO, sprintf("LoopDetection->ProcessLoopDetectionIsHierarchySyncAdvised(): Ping loop of folderid '%s' detected that is not being synchronized.", $fid));
+                    return true;
+                }
+                // Sync is executed, but a foldersync should be executed (hierarchy errors)
+                if (isset($data[ZPush::COMMAND_SYNC][SYNC_STATUS_FOLDERHIERARCHYCHANGED]) &&
+                        $data[ZPush::COMMAND_SYNC][SYNC_STATUS_FOLDERHIERARCHYCHANGED] > 3 ) {
+                    ZLog::Write(LOGLEVEL_INFO, sprintf("LoopDetection->ProcessLoopDetectionIsHierarchySyncAdvised(): Sync(with error)/Ping loop of folderid '%s' detected.", $fid));
+                    return true;
+                }
+            }
+        }
+
+        return false;
+    }
+
     /**
      * Indicates if a full Hierarchy Resync is necessary
      *
@@ -220,6 +269,11 @@ class LoopDetection extends InterProcessData {
                 // look for sync command
                 if (isset($se['stat']) && ($se['cc'] == ZPush::COMMAND_SYNC || $se['cc'] == ZPush::COMMAND_PING)) {
                     foreach($se['stat'] as $key => $value) {
+                        // don't count PING with changes on a folder or sync with success
+                        if (($se['cc'] == ZPush::COMMAND_PING && $value == SYNC_PINGSTATUS_CHANGES) ||
+                            ($se['cc'] == ZPush::COMMAND_SYNC && $value == SYNC_STATUS_SUCCESS) ) {
+                            continue;
+                        }
                         if (!isset($seenFailed[$key]))
                             $seenFailed[$key] = 0;
                         $seenFailed[$key]++;
@@ -610,6 +664,7 @@ class LoopDetection extends InterProcessData {
      *    2. request counter is the same as the previous, but no data was sent on the last request (standard situation)
      *
      *    3. request counter is the same as the previous and last time objects were sent (loop!)
+     *      3.0)   no loop was detected before, but with big window size -> lower window size first - NO LOOP mode yet
      *      3.1)   no loop was detected before, entereing loop mode     -> save loop data, loopcount = 1
      *      3.2)   loop was detected before, but are gone               -> loop resolved
      *      3.3)   loop was detected before, continuing in loop mode    -> this is probably the broken element,loopcount++,
@@ -622,7 +677,7 @@ class LoopDetection extends InterProcessData {
      * @param string $queuedMessages    the amount of messages which were found by the exporter
      *
      * @access public
-     * @return boolean      when returning true if a loop has been identified
+     * @return boolean/int      when returning true if a loop has been identified - returns new suggested window size if window might have been too big
      */
     public function Detect($folderid, $uuid, $counter, $maxItems, $queuedMessages) {
         ZLog::Write(LOGLEVEL_DEBUG, sprintf("LoopDetection->Detect(): folderid:'%s' uuid:'%s' counter:'%s' max:'%s' queued:'%s'", $folderid, $uuid, $counter, $maxItems, $queuedMessages));
@@ -676,6 +731,7 @@ class LoopDetection extends InterProcessData {
                 unset($current['ignored']);
                 unset($current['maxCount']);
                 unset($current['potential']);
+                unset($current['windowLimit']);
             }
 
             // see if there are values
@@ -709,6 +765,7 @@ class LoopDetection extends InterProcessData {
                             unset($current['ignored']);
                             unset($current['maxCount']);
                             unset($current['potential']);
+                            unset($current['windowLimit']);
                         }
                     }
                 }
@@ -724,12 +781,28 @@ class LoopDetection extends InterProcessData {
                 else if ($current['count'] == $counter && $current['queued'] > 0) {
 
                     if (!isset($current['loopcount'])) {
-                        // case 3.1) we have just encountered a loop!
-                        ZLog::Write(LOGLEVEL_DEBUG, "LoopDetection->Detect(): case 3.1 detected - loop detected, init loop mode");
-                        $current['loopcount'] = 1;
-                        // the MaxCount is the max number of messages exported before
-                        $current['maxCount'] = $counter + (($maxItems < $queuedMessages) ? $maxItems : $queuedMessages);
-                        $loop = true;   // loop mode!!
+                        // ZP-1213 we are potentially synching a lot of data, e.g. OL with 512 WindowSize
+                        // In case there are more then 40 items in the last request, we limit to 25 items
+                        // before entering 1-by-1 loop detection if counter is re-requested
+                        if ($maxItems > 40 && !isset($current['windowLimit'])) {
+                            // case 3.0) we have just encountered a loop, but with a big window size, lower window first
+                            ZLog::Write(LOGLEVEL_DEBUG, sprintf("LoopDetection->Detect(): case 3.0 detected - big windowsize of %d, lowering before entering loop mode", $maxItems));
+                            // return suggested new window size
+                            $current['windowLimit'] = 25;
+                            $loop = $current['windowLimit'];
+                        }
+                        else {
+                            // case 3.1) we have just encountered a loop!
+                            ZLog::Write(LOGLEVEL_DEBUG, "LoopDetection->Detect(): case 3.1 detected - loop detected, init loop mode");
+                            if (isset($current['windowLimit'])) {
+                                $maxItems = $current['windowLimit'];
+                                unset($current['windowLimit']);
+                            }
+                            $current['loopcount'] = 1;
+                            // the MaxCount is the max number of messages exported before
+                            $current['maxCount'] = $counter + (($maxItems < $queuedMessages) ? $maxItems : $queuedMessages);
+                            $loop = true;   // loop mode!!
+                        }
                     }
                     else if ($queuedMessages == 0) {
                         // case 3.2) there was a loop before but now the changes are GONE
@@ -739,6 +812,7 @@ class LoopDetection extends InterProcessData {
                         unset($current['ignored']);
                         unset($current['maxCount']);
                         unset($current['potential']);
+                        unset($current['windowLimit']);
                     }
                     else {
                         // case 3.3) still looping the same message! Increase counter
@@ -767,7 +841,7 @@ class LoopDetection extends InterProcessData {
         }
         // end exclusive block
 
-        if ($loop == true && $this->ignore_messageid == false) {
+        if ($loop === true && $this->ignore_messageid == false) {
             ZPush::GetTopCollector()->AnnounceInformation("Loop detection", true);
         }
 
