@@ -12,25 +12,7 @@
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3,
-* as published by the Free Software Foundation with the following additional
-* term according to sec. 7:
-*
-* According to sec. 7 of the GNU Affero General Public License, version 3,
-* the terms of the AGPL are supplemented with the following terms:
-*
-* "Zarafa" is a registered trademark of Zarafa B.V.
-* "Z-Push" is a registered trademark of Zarafa Deutschland GmbH
-* The licensing of the Program under the AGPL does not imply a trademark license.
-* Therefore any rights, title and interest in our trademarks remain entirely with us.
-*
-* However, if you propagate an unmodified version of the Program you are
-* allowed to use the term "Z-Push" to indicate that you distribute the Program.
-* Furthermore you may use our trademarks where it is necessary to indicate
-* the intended purpose of a product or service provided you use it in accordance
-* with honest practices in industrial or commercial matters.
-* If you want to propagate modified versions of the Program under the name "Z-Push",
-* you may only do so if you have a written permission by Zarafa Deutschland GmbH
-* (to acquire a permission please contact Zarafa at trademark@zarafa.com).
+* as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -82,7 +64,7 @@ class SyncParameters extends StateObject {
     /**
      * SyncParameters constructor
      */
-    public function SyncParameters() {
+    public function __construct() {
         // initialize ContentParameters for the current option
         $this->checkCPO();
     }
@@ -304,6 +286,17 @@ class SyncParameters extends StateObject {
         $this->checkCPO($this->currentCPO);
     }
 
+
+    /**
+     * Indicates if the confirmation status changed for the SyncKey.
+     *
+     * @access public
+     * @return boolean
+     */
+    public function HasConfirmationChanged() {
+        return $this->confirmationChanged;
+    }
+
     /**
      * Indicates if a exporter run is required. This is the case if the given folderstat is different from the saved one
      * or when the expiration time expired.
