@@ -33,6 +33,9 @@ class FileStateMachine implements IStateMachine {
 
     private $userfilename;
     private $settingsfilename;
+    private $statefiles; // List of the state files. Used by z-push-admin and scripts.
+    private $devicedatafiles; // List of the device data files. Used by z-push-admin and scripts.
+    private $pattern; // State pattern for glob()
 
     /**
      * Constructor
@@ -59,6 +62,10 @@ class FileStateMachine implements IStateMachine {
         if ((!file_exists($this->userfilename) && !touch($this->userfilename)) || !is_writable($this->userfilename))
             throw new FatalMisconfigurationException("Not possible to write to the configured state directory.");
         Utils::FixFileOwner($this->userfilename);
+
+        $this->statefiles = array();
+        $this->devicedatafiles = array();
+        $this->pattern = STATE_DIR.'*/*/*';
     }
 
     /**
@@ -109,7 +116,7 @@ class FileStateMachine implements IStateMachine {
         $filename = $this->getFullFilePath($devid, $type, $key, $counter);
 
         if(file_exists($filename)) {
-            $contents = file_get_contents($filename);
+            $contents = Utils::SafeGetContents($filename, __FUNCTION__, false);
             $bytes = strlen($contents);
             ZLog::Write(LOGLEVEL_DEBUG, sprintf("FileStateMachine->GetState() read '%d' bytes from file: '%s'", $bytes, $filename ));
             return unserialize($contents);
@@ -160,7 +167,7 @@ class FileStateMachine implements IStateMachine {
      * @throws StateInvalidException
      */
     public function CleanStates($devid, $type, $key, $counter = false, $thisCounterOnly = false) {
-        $matching_files = glob($this->getFullFilePath($devid, $type, $key). "*", GLOB_NOSORT);
+        $matching_files = $this->getStateFiles($this->getFullFilePath($devid, $type, $key). "*");
         if (is_array($matching_files)) {
             foreach($matching_files as $state) {
                 $file = false;
@@ -198,7 +205,7 @@ class FileStateMachine implements IStateMachine {
 
         // exclusive block
         if ($mutex->Block()) {
-            $filecontents = @file_get_contents($this->userfilename);
+            $filecontents = Utils::SafeGetContents($this->userfilename, __FUNCTION__, true);
 
             if ($filecontents)
                 $users = unserialize($filecontents);
@@ -216,7 +223,7 @@ class FileStateMachine implements IStateMachine {
             }
 
             if ($changed) {
-                $bytes = Utils::SafePutContents($this->userfilename, serialize($users));
+                $bytes = Utils::SafePutContents($this->userfilename, serialize($users), true);
                 ZLog::Write(LOGLEVEL_DEBUG, sprintf("FileStateMachine->LinkUserDevice(): wrote %d bytes to users file", $bytes));
             }
             else
@@ -242,7 +249,7 @@ class FileStateMachine implements IStateMachine {
 
         // exclusive block
         if ($mutex->Block()) {
-            $filecontents = @file_get_contents($this->userfilename);
+            $filecontents = Utils::SafeGetContents($this->userfilename, __FUNCTION__, true);
 
             if ($filecontents)
                 $users = unserialize($filecontents);
@@ -264,7 +271,7 @@ class FileStateMachine implements IStateMachine {
             }
 
             if ($changed) {
-                $bytes = Utils::SafePutContents($this->userfilename, serialize($users));
+                $bytes = Utils::SafePutContents($this->userfilename, serialize($users), true);
                 ZLog::Write(LOGLEVEL_DEBUG, sprintf("FileStateMachine->UnLinkUserDevice(): wrote %d bytes to users file", $bytes));
             }
             else
@@ -287,13 +294,14 @@ class FileStateMachine implements IStateMachine {
     public function GetAllDevices($username = false) {
         $out = array();
         if ($username === false) {
-            foreach (glob(STATE_DIR. "/*/*/*-".IStateMachine::DEVICEDATA, GLOB_NOSORT) as $devdata)
+            foreach ($this->getDeviceDataFiles() as $devdata)
+                // TODO do we still need this check here?
                 if (preg_match('/\/([A-Za-z0-9]+)-'. IStateMachine::DEVICEDATA. '$/', $devdata, $matches))
                     $out[] = $matches[1];
             return $out;
         }
         else {
-            $filecontents = file_get_contents($this->userfilename);
+            $filecontents = Utils::SafeGetContents($this->userfilename, __FUNCTION__, false);
             if ($filecontents)
                 $users = unserialize($filecontents);
             else
@@ -315,7 +323,8 @@ class FileStateMachine implements IStateMachine {
      */
     public function GetStateVersion() {
         if (file_exists($this->settingsfilename)) {
-            $settings = unserialize(file_get_contents($this->settingsfilename));
+            $filecontents = Utils::SafeGetContents($this->settingsfilename, __FUNCTION__, false);
+            $settings = unserialize($filecontents);
             if (strtolower(gettype($settings) == "string") && strtolower($settings) == '2:1:{s:7:"version";s:1:"2";}') {
                 ZLog::Write(LOGLEVEL_INFO, "Broken state version file found. Attempt to autofix it. See https://jira.zarafa.com/browse/ZP-493 for more information.");
                 unlink($this->settingsfilename);
@@ -324,7 +333,7 @@ class FileStateMachine implements IStateMachine {
             }
         }
         else {
-            $filecontents = @file_get_contents($this->userfilename);
+            $filecontents = Utils::SafeGetContents($this->userfilename, __FUNCTION__, true);
             if ($filecontents)
                 $settings = array(self::VERSION => IStateMachine::STATEVERSION_01);
             else {
@@ -345,8 +354,10 @@ class FileStateMachine implements IStateMachine {
      * @return boolean
      */
     public function SetStateVersion($version) {
-        if (file_exists($this->settingsfilename))
-            $settings = unserialize(file_get_contents($this->settingsfilename));
+        if (file_exists($this->settingsfilename)){
+            $filecontents = Utils::SafeGetContents($this->settingsfilename, __FUNCTION__, false);
+            $settings = unserialize($filecontents);
+        }
         else
             $settings = array(self::VERSION => IStateMachine::STATEVERSION_01);
 
@@ -368,8 +379,8 @@ class FileStateMachine implements IStateMachine {
         $types = array(IStateMachine::DEVICEDATA, IStateMachine::FOLDERDATA, IStateMachine::FAILSAVE, IStateMachine::HIERARCHY, IStateMachine::BACKENDSTORAGE);
         $out = array();
         $devdir = $this->getDirectoryForDevice($devid) . "/$devid-";
-
-        foreach (glob($devdir . "*", GLOB_NOSORT) as $devdata) {
+        $deviceFiles = array_filter($this->getStateFiles(), function($var) use ($devdir) {return strpos($var, $devdir) !== false;});
+        foreach ($deviceFiles as $devdata) {
             // cut the device dir away and split into parts
             $parts = explode("-", substr($devdata, strlen($devdir)));
 
@@ -379,11 +390,10 @@ class FileStateMachine implements IStateMachine {
             if (isset($parts[0]) && in_array($parts[0], $types))
                 $state['type'] = $parts[0];
 
-            if (isset($parts[0]) && strlen($parts[0]) == 8 &&
-                isset($parts[1]) && strlen($parts[1]) == 4 &&
-                isset($parts[2]) && strlen($parts[2]) == 4 &&
-                isset($parts[3]) && strlen($parts[3]) == 4 &&
-                isset($parts[4]) && strlen($parts[4]) == 12) {
+            if (isset($parts[0], $parts[1], $parts[2], $parts[3], $parts[4]) &&
+                    strlen($parts[0]) == 8 && strlen($parts[1]) == 4 &&
+                    strlen($parts[2]) == 4 && strlen($parts[3]) == 4 &&
+                    strlen($parts[4]) == 12) {
 
                 $state['uuid'] = $parts[0]."-".$parts[1]."-".$parts[2]."-".$parts[3]."-".$parts[4];
             }
@@ -482,4 +492,36 @@ class FileStateMachine implements IStateMachine {
         return false;
     }
 
+    /**
+     * Returns the list of the state files.
+     *
+     * @param string    $pattern state pattern for glob()
+     *
+     * @access public
+     * @return array
+     */
+    protected function getStateFiles($pattern = null) {
+        if ($pattern === null || $pattern === $this->pattern) {
+            if (empty($this->statefiles)) {
+                $this->statefiles = glob($this->pattern, GLOB_NOSORT);
+                ZLog::Write(LOGLEVEL_DEBUG, sprintf("FileStateMachine->getStateFiles() read all state files '%d'", sizeof($this->statefiles)));
+            }
+            return $this->statefiles;
+        }
+        ZLog::Write(LOGLEVEL_DEBUG, sprintf("FileStateMachine->getStateFiles() reading state files of '%s'", $pattern));
+        return glob($pattern, GLOB_NOSORT);
+    }
+
+    /**
+     * Filters the list of the state files and returns the device data files only.
+     *
+     * @access public
+     * @return array
+     */
+    protected function getDeviceDataFiles() {
+        if (empty($this->devicedatafiles)) {
+            $this->devicedatafiles = array_filter($this->GetStateFiles(), function($var) {return strpos($var, IStateMachine::DEVICEDATA) !== false;} );
+        }
+        return $this->devicedatafiles;
+    }
 }
