@@ -14,25 +14,7 @@
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3,
-* as published by the Free Software Foundation with the following additional
-* term according to sec. 7:
-*
-* According to sec. 7 of the GNU Affero General Public License, version 3,
-* the terms of the AGPL are supplemented with the following terms:
-*
-* "Zarafa" is a registered trademark of Zarafa B.V.
-* "Z-Push" is a registered trademark of Zarafa Deutschland GmbH
-* The licensing of the Program under the AGPL does not imply a trademark license.
-* Therefore any rights, title and interest in our trademarks remain entirely with us.
-*
-* However, if you propagate an unmodified version of the Program you are
-* allowed to use the term "Z-Push" to indicate that you distribute the Program.
-* Furthermore you may use our trademarks where it is necessary to indicate
-* the intended purpose of a product or service provided you use it in accordance
-* with honest practices in industrial or commercial matters.
-* If you want to propagate modified versions of the Program under the name "Z-Push",
-* you may only do so if you have a written permission by Zarafa Deutschland GmbH
-* (to acquire a permission please contact Zarafa at trademark@zarafa.com).
+* as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -45,7 +27,6 @@
 * Consult LICENSE file for details
 ************************************************/
 
-
 class SyncProvisioning extends SyncObject {
     //AS 12.0, 12.1 and 14.0 props
     public $devpwenabled;
@@ -97,7 +78,7 @@ class SyncProvisioning extends SyncObject {
     // policy name used with the policies; not part of ActiveSync
     public $PolicyName;
 
-    function SyncProvisioning() {
+    function __construct() {
         $mapping = array (
                     SYNC_PROVISION_DEVPWENABLED                         => array (  self::STREAMER_VAR      => "devpwenabled",
                                                                                     self::STREAMER_CHECKS   => array(   self::STREAMER_CHECK_ONEVALUEOF => array(0,1) )),
@@ -239,7 +220,7 @@ class SyncProvisioning extends SyncObject {
             );
         }
 
-        parent::SyncObject($mapping);
+        parent::__construct($mapping);
     }
 
     /**
@@ -279,12 +260,11 @@ class SyncProvisioning extends SyncObject {
         $this->alphanumpwreq = 0;
         $this->devencenabled = 0;
         $this->pwrecoveryenabled = 0;
-        $this->docbrowseenabled;
         $this->attenabled = 1;
         $this->mindevpwlenngth = 4;
         $this->maxinacttimedevlock = 900;
         $this->maxdevpwfailedattempts = 8;
-        $this->maxattsize = '';
+        $this->maxattsize = -1;
         $this->allowsimpledevpw = 1;
         $this->devpwexpiration = 0;
         $this->devpwhistory = 0;
