@@ -1178,7 +1178,7 @@ class TimezoneUtil {
     }
 
     /**
-     * Tries to find a AS timezone for a php timezone
+     * Tries to find a AS timezone for a php timezone.
      *
      * @param string    $phpname        a php timezone name
      *
@@ -1188,17 +1188,14 @@ class TimezoneUtil {
     static private function guessTZNameFromPHPName($phpname) {
         foreach (self::$phptimezones as $tzn => $phptzs) {
             if (in_array($phpname, $phptzs)) {
-                $tzname = $tzn;
+                if (!is_int($tzn)) {
+                    return $tzn;
+                }
                 break;
             }
         }
-
-        if (!isset($tzname) || is_int($tzname)) {
-            ZLog::Write(LOGLEVEL_ERROR, sprintf("TimezoneUtil::guessTZNameFromPHPName() no compatible timezone found for '%s'. Returning 'GMT Standard Time'. Please contact the Z-Push dev team.", $phpname));
-            return self::$mstzones["085"][0];
-        }
-
-        return $tzname;
+        ZLog::Write(LOGLEVEL_ERROR, sprintf("TimezoneUtil::guessTZNameFromPHPName() no compatible timezone found for '%s'. Returning 'GMT Standard Time'. Please contact the Z-Push dev team.", $phpname));
+        return self::$mstzones["085"][0];
     }
 
     /**
@@ -1359,4 +1356,47 @@ class TimezoneUtil {
         }
         return self::getMSTZnameFromTZName(trim($timezone, '"'));
     }
+
+    /**
+     * Returns a timezone supported by PHP for DateTimeZone constructor.
+     * @see http://php.net/manual/en/timezones.php
+     *
+     * @param string $timezone
+     *
+     * @access public
+     * @return string
+     */
+    public static function GetPhpSupportedTimezone($timezone) {
+        if (in_array($timezone, DateTimeZone::listIdentifiers())) {
+            ZLog::Write(LOGLEVEL_DEBUG, sprintf("TimezoneUtil::GetPhpSupportedTimezone(): '%s' is a PHP supported timezone", $timezone));
+            return $timezone;
+        }
+        $dtz = date_default_timezone_get();
+        ZLog::Write(LOGLEVEL_DEBUG, sprintf("TimezoneUtil::GetPhpSupportedTimezone(): '%s' is not a PHP supported timezone. Returning default timezone: '%s'", $timezone, $dtz));
+        return $dtz;
+    }
+
+    /**
+     * Returns official timezone name from windows timezone name.
+     * E.g. "W Europe Standard Time" for "(GMT+01:00) Amsterdam, Berlin, Bern, Rome, Stockholm, Vienna".
+     *
+     * @param string $winTz Timezone name in windows
+     *
+     * @access public
+     * @return string timezone name
+     */
+    public static function GetTZNameFromWinTZ($winTz = false) {
+
+        // Return "GMT Standard Time" per default
+        if ($winTz === false) {
+            return self::$mstzones['085'][0];
+        }
+
+        foreach (self::$mstzones as $mskey => $msdefs) {
+            if ($winTz == $msdefs[1]) {
+                return $msdefs[0];
+            }
+        }
+        return self::$mstzones['085'][0];
+    }
 }
