@@ -33,6 +33,7 @@ class ZPushAdmin {
 
     public static $status = self::STATUS_SUCCESS;
     public static $devices;
+    public static $userDevices;
 
     /**
      * List devices known to Z-Push.
@@ -818,18 +819,35 @@ class ZPushAdmin {
     }
 
     /**
-     * Fixes states with usernames in different cases
+     * Fixes states with usernames in different cases.
+     *
+     * @param string    $username . Has precedence over $deviceId
+     * @param string    $deviceId
      *
      * @return boolean
      * @access public
      */
-    static public function FixStatesDifferentUsernameCases() {
+    static public function FixStatesDifferentUsernameCases($username=false, $deviceId=false) {
         $processed = 0;
         $dropedUsers = 0;
         $fixedUsers = 0;
+        $processedDevices = 0;
 
-        $devices = self::GetAllDevices();
+        if ($username) {
+            $devices = self::GetUserDevices($username);
+        }
+        else if ($deviceId){
+            $devices = array($deviceId);
+        }
+        else {
+            $devices = self::GetAllDevices();
+        }
+        $devicesCount = count($devices);
+        ZLog::Write(LOGLEVEL_DEBUG, sprintf("ZPushAdmin::FixStatesDifferentUsernameCases(): found %d devices", $devicesCount));
         foreach ($devices as $devid) {
+            if (defined('LOGFIXSTATES') && LOGFIXSTATES === true) {
+                ZLog::Write(LOGLEVEL_INFO, sprintf("ZPushAdmin::FixStatesDifferentUsernameCases(): Processing %d of %d. Device %s", ++$processedDevices, $devicesCount, $devid));
+            }
             $users = self::ListUsers($devid);
             $obsoleteUsers = array();
 
@@ -889,18 +907,34 @@ class ZPushAdmin {
     }
 
     /**
-     * Fixes states of available device data to the user linking
+     * Fixes states of available device data to the user linking.
+     *
+     * @param string    $username . Has precedence over $deviceId
+     * @param string    $deviceId
      *
      * @return int
      * @access public
      */
-    static public function FixStatesDeviceToUserLinking() {
+    static public function FixStatesDeviceToUserLinking($username=false, $deviceId=false) {
         $seen = 0;
         $fixed = 0;
-        $devices = self::GetAllDevices();
-        ZLog::Write(LOGLEVEL_DEBUG, sprintf("ZPushAdmin::FixStatesDeviceToUserLinking(): found %d devices", count($devices)));
+        $processedDevices = 0;
 
+        if ($username) {
+            $devices = self::GetUserDevices($username);
+        }
+        else if ($deviceId){
+            $devices = array($deviceId);
+        }
+        else {
+            $devices = self::GetAllDevices();
+        }
+        $devicesCount = count($devices);
+        ZLog::Write(LOGLEVEL_DEBUG, sprintf("ZPushAdmin::FixStatesDeviceToUserLinking(): found %d devices", $devicesCount));
         foreach ($devices as $devid) {
+            if (defined('LOGFIXSTATES') && LOGFIXSTATES === true) {
+                ZLog::Write(LOGLEVEL_INFO, sprintf("ZPushAdmin::FixStatesDeviceToUserLinking(): Processing %d of %d. Device %s", ++$processedDevices, $devicesCount, $devid));
+            }
             $users = self::ListUsers($devid);
             foreach ($users as $username) {
                 $seen++;
@@ -915,19 +949,36 @@ class ZPushAdmin {
 
     /**
      * Fixes states of the user linking to the states
-     * and removes all obsolete states
+     * and removes all obsolete states.
+     *
+     * @param string    $username . Has precedence over $deviceId
+     * @param string    $deviceId
      *
      * @return boolean
      * @access public
      */
-    static public function FixStatesUserToStatesLinking() {
+    static public function FixStatesUserToStatesLinking($username=false, $deviceId=false) {
         $processed = 0;
         $deleted = 0;
-        $devices = self::GetAllDevices();
-        ZLog::Write(LOGLEVEL_DEBUG, sprintf("ZPushAdmin::FixStatesUserToStatesLinking(): found %d devices", count($devices)));
-
+        $processedDevices = 0;
+        $processedStates = 0;
+        $deletedStates = 0;
+        if ($username) {
+            $devices = self::GetUserDevices($username);
+        }
+        else if ($deviceId){
+            $devices = array($deviceId);
+        }
+        else {
+            $devices = self::GetAllDevices();
+        }
+        $devicesCount = count($devices);
+        ZLog::Write(LOGLEVEL_DEBUG, sprintf("ZPushAdmin::FixStatesUserToStatesLinking(): found %d devices", $devicesCount));
         foreach ($devices as $devid) {
             try {
+                if (defined('LOGFIXSTATES') && LOGFIXSTATES === true) {
+                    ZLog::Write(LOGLEVEL_INFO, sprintf("ZPushAdmin::FixStatesUserToStatesLinking(): Processing %d of %d. Device %s", ++$processedDevices, $devicesCount, $devid));
+                }
                 // we work on device level
                 $devicedata = ZPush::GetStateMachine()->GetState($devid, IStateMachine::DEVICEDATA);
                 $knownUuids = array();
@@ -956,9 +1007,13 @@ class ZPushAdmin {
             // get all uuids for deviceid from statemachine
             $existingStates = ZPush::GetStateMachine()->GetAllStatesForDevice($devid);
             $processed = count($existingStates);
-
-            ZLog::Write(LOGLEVEL_DEBUG, sprintf("ZPushAdmin::FixStatesUserToStatesLinking(): found %d valid uuids and %d states for device '%s'", count($knownUuids), $processed, $devid));
-
+            $deleted = 0;
+            if (defined('LOGFIXSTATES') && LOGFIXSTATES === true) {
+                ZLog::Write(LOGLEVEL_INFO, sprintf("ZPushAdmin::FixStatesUserToStatesLinking(): found %d valid uuids and %d states for device '%s'", count($knownUuids), $processed, $devid));
+            }
+            else {
+                ZLog::Write(LOGLEVEL_DEBUG, sprintf("ZPushAdmin::FixStatesUserToStatesLinking(): found %d valid uuids and %d states for device '%s'", count($knownUuids), $processed, $devid));
+            }
             // remove states for all unknown uuids
             foreach ($existingStates as $obsoleteState) {
                 if ($obsoleteState['type'] === IStateMachine::DEVICEDATA)
@@ -967,31 +1022,51 @@ class ZPushAdmin {
                 if (!in_array($obsoleteState['uuid'], $knownUuids)) {
                     if (is_numeric($obsoleteState['counter']))
                         $obsoleteState['counter']++;
-
+                    if (defined('LOGFIXSTATES') && LOGFIXSTATES === true) {
+                        ZLog::Write(LOGLEVEL_INFO, sprintf("ZPushAdmin::FixStatesUserToStatesLinking(): obsoleteState TYPE '%s' UUID '%s' COUNTER %d for device '%s'", $obsoleteState['type'], $obsoleteState['uuid'], $obsoleteState['counter'], $devid));
+                    }
                     ZPush::GetStateMachine()->CleanStates($devid, $obsoleteState['type'], $obsoleteState['uuid'], $obsoleteState['counter']);
                     $deleted++;
                 }
             }
+            $processedStates +=  $processed;
+            $deletedStates += $deleted;
         }
-        return array($processed, $deleted);
+        return array($processedStates, $deletedStates);
     }
 
     /**
      * Fixes hierarchy states writing folderdata states.
      *
+     * @param string    $username . Has precedence over $deviceId
+     * @param string    $deviceId
+     *
      * @access public
      * @return array(seenDevices, seenHierarchyStates, fixedHierarchyStates, usersWithoutHierarchy)
      */
-    static public function FixStatesHierarchyFolderData() {
+    static public function FixStatesHierarchyFolderData($username=false, $deviceId=false) {
         $devices = 0;
         $seen = 0;
         $nouuid = 0;
         $fixed = 0;
-        $asdevices = self::GetAllDevices();
-        ZLog::Write(LOGLEVEL_DEBUG, sprintf("ZPushAdmin::FixStatesHierarchyFolderData(): found %d devices", count($asdevices)));
+        $processedDevices = 0;
 
+        if ($username) {
+            $asdevices = self::GetUserDevices($username);
+        }
+        else if ($deviceId){
+            $asdevices = array($deviceId);
+        }
+        else {
+            $asdevices = self::GetAllDevices();
+        }
+        $devicesCount = count($asdevices);
+        ZLog::Write(LOGLEVEL_DEBUG, sprintf("ZPushAdmin::FixStatesHierarchyFolderData(): found %d devices", $devicesCount));
         foreach ($asdevices as $devid) {
             try {
+                if (defined('LOGFIXSTATES') && LOGFIXSTATES === true) {
+                    ZLog::Write(LOGLEVEL_INFO, sprintf("ZPushAdmin::FixStatesHierarchyFolderData(): Processing %d of %d. Device %s", ++$processedDevices, $devicesCount, $devid));
+                }
                 // get the device
                 $devicedata = ZPush::GetStateMachine()->GetState($devid, IStateMachine::DEVICEDATA);
                 $devices++;
@@ -1018,6 +1093,7 @@ class ZPushAdmin {
                     }
                     catch(StateNotFoundException $snfe) {
                         $needsFixing = true;
+                        ZLog::Write(LOGLEVEL_INFO, sprintf("ZPushAdmin::FixStatesHierarchyFolderData(): device %s user '%s' needs fixing.", $devid, $username));
                     }
                     // Search all states, and find the highest counter for the hierarchy UUID
                     $allStates = ZPush::GetStateMachine()->GetAllStatesForDevice($devid);
@@ -1030,8 +1106,10 @@ class ZPushAdmin {
                     $hierarchySyncKey = StateManager::BuildStateKey($hierarchyUuid, $maxCounter);
 
                     if ($spa) {
-                        if ($spa->GetSyncKey() !== $hierarchySyncKey) {
+                        //Don't fix if $spa->GetUuidCounter() > $maxCounter
+                        if ($spa->GetSyncKey() !== $hierarchySyncKey && $spa->GetUuidCounter() < $maxCounter) {
                             $needsFixing = true;
+                            ZLog::Write(LOGLEVEL_INFO, sprintf("ZPushAdmin::FixStatesHierarchyFolderData(): device %s user '%s' needs fixing. spa->GetSyncKey()='%s' hierarchySyncKey='%s'.", $devid, $username, $spa->GetSyncKey(), $hierarchySyncKey));
                         }
                     }
                     else {
@@ -1042,7 +1120,7 @@ class ZPushAdmin {
                         // generate FOLDERDATA
                         $spa->SetSyncKey($hierarchySyncKey);
                         $spa->SetFolderId(false);
-                        ZLog::Write(LOGLEVEL_DEBUG, sprintf("ZPushAdmin::FixStatesHierarchyFolderData(): write data for %s", $spa->GetSyncKey()));
+                        ZLog::Write(LOGLEVEL_INFO, sprintf("ZPushAdmin::FixStatesHierarchyFolderData(): device %s user '%s' needs fixing. write data for %s", $devid, $username, $spa->GetSyncKey()));
                         ZPush::GetStateMachine()->SetState($spa, $device->GetDeviceId(), IStateMachine::FOLDERDATA, $hierarchyUuid);
                         $fixed++;
                     }
@@ -1056,18 +1134,34 @@ class ZPushAdmin {
     /**
      * Fixes missing flags or parentids on additional folders.
      *
+     * @param string    $username . Has precedence over $deviceId
+     * @param string    $deviceId
+     *
      * @access public
      * @return array(seenDevices, devicesWithAdditionalFolders, fixedAdditionalFolders)
      */
-    static public function FixStatesAdditionalFolders() {
+    static public function FixStatesAdditionalFolders($username=false, $deviceId=false) {
         $devices = 0;
         $devicesWithAddFolders = 0;
         $fixed = 0;
-        $asdevices = self::GetAllDevices();
-        ZLog::Write(LOGLEVEL_DEBUG, sprintf("ZPushAdmin::FixStatesAdditionalFolders(): found %d devices", count($asdevices)));
+        $processedDevices = 0;
 
+        if ($username) {
+            $asdevices = self::GetUserDevices($username);
+        }
+        else if ($deviceId){
+            $asdevices = array($deviceId);
+        }
+        else {
+            $asdevices = self::GetAllDevices();
+        }
+        $devicesCount = count($asdevices);
+        ZLog::Write(LOGLEVEL_DEBUG, sprintf("ZPushAdmin::FixStatesAdditionalFolders(): found %d devices", $devicesCount));
         foreach ($asdevices as $devid) {
             try {
+                if (defined('LOGFIXSTATES') && LOGFIXSTATES === true) {
+                    ZLog::Write(LOGLEVEL_INFO, sprintf("ZPushAdmin::FixStatesAdditionalFolders(): Processing %d of %d. Device %s", ++$processedDevices, $devicesCount, $devid));
+                }
                 // get the device
                 $devicedata = ZPush::GetStateMachine()->GetState($devid, IStateMachine::DEVICEDATA);
                 $devices++;
@@ -1119,8 +1213,50 @@ class ZPushAdmin {
     public static function GetAllDevices() {
         if (empty(self::$devices)) {
             self::$devices = ZPush::GetStateMachine()->GetAllDevices(false);
+            if (defined('LOGFIXSTATES') && LOGFIXSTATES === true) {
+                ZLog::Write(LOGLEVEL_INFO, sprintf("ZPushAdmin::GetAllDevices(): found %d devices", count(self::$devices)));
+            }
         }
         return self::$devices;
     }
 
+     /**
+     * Returns the list of all devices of a specific user.
+     *
+     * @access public
+     * @return array
+     */
+    public static function GetUserDevices($username) {
+        if (empty(self::$userDevices) && empty(self::$devices)) {
+            if (defined('LOGFIXSTATES') && LOGFIXSTATES === true){
+                ZLog::Write(LOGLEVEL_INFO, sprintf("ZPushAdmin::GetUserDevices(): Getting all devices for '%s'", $username));
+            }
+            $devices = self::GetAllDevices();
+            $devicesCount = count($devices);
+            self::$userDevices = array();
+            $processed = 0;
+
+            foreach ($devices as $devid) {
+                if (defined('LOGFIXSTATES') && LOGFIXSTATES === true){
+                    ZLog::Write(LOGLEVEL_INFO, sprintf("ZPushAdmin::GetUserDevices(): Processing %d on %d . Device %s", ++$processed , $devicesCount, $devid));
+                }
+                $users = self::ListUsers($devid);
+                foreach ($users as $user) {
+                    if ($user === $username){
+                        ZLog::Write(LOGLEVEL_INFO, sprintf("ZPushAdmin::GetUserDevices(): Found user '%s' into device %s", $username , $devid));
+                        self::$userDevices[] = $devid;
+                        continue;
+                    }
+                }
+            }
+
+            if (empty(self::$userDevices) && $username!==false && defined('LOGFIXSTATES') && LOGFIXSTATES === true){
+                ZLog::Write(LOGLEVEL_INFO, sprintf("ZPushAdmin::GetUserDevices(): No devices found for '%s'", $username));
+            }else if(defined('LOGFIXSTATES') && LOGFIXSTATES === true){
+                ZLog::Write(LOGLEVEL_INFO, sprintf("ZPushAdmin::GetUserDevices(): Found user '%s' into '%d' devices", $username , count(self::$userDevices)));
+            }
+        }
+        return self::$userDevices;
+    }
+
 }
