@@ -6,29 +6,11 @@
 *
 * Created   :   01.10.2007
 *
-* Copyright 2007 - 2015 Zarafa Deutschland GmbH
+* Copyright 2007 - 2016 Zarafa Deutschland GmbH
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3,
-* as published by the Free Software Foundation with the following additional
-* term according to sec. 7:
-*
-* According to sec. 7 of the GNU Affero General Public License, version 3,
-* the terms of the AGPL are supplemented with the following terms:
-*
-* "Zarafa" is a registered trademark of Zarafa B.V.
-* "Z-Push" is a registered trademark of Zarafa Deutschland GmbH
-* The licensing of the Program under the AGPL does not imply a trademark license.
-* Therefore any rights, title and interest in our trademarks remain entirely with us.
-*
-* However, if you propagate an unmodified version of the Program you are
-* allowed to use the term "Z-Push" to indicate that you distribute the Program.
-* Furthermore you may use our trademarks where it is necessary to indicate
-* the intended purpose of a product or service provided you use it in accordance
-* with honest practices in industrial or commercial matters.
-* If you want to propagate modified versions of the Program under the name "Z-Push",
-* you may only do so if you have a written permission by Zarafa Deutschland GmbH
-* (to acquire a permission please contact Zarafa at trademark@zarafa.com).
+* as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -41,7 +23,6 @@
 * Consult LICENSE file for details
 ************************************************/
 
-
 class WBXMLDecoder extends WBXMLDefs {
     private $in;
 
@@ -75,13 +56,13 @@ class WBXMLDecoder extends WBXMLDefs {
         else {
             self::$loopCounter[$name]++;
         }
-    
+
         if (self::$loopCounter[$name] > self::MAXLOOP) {
             throw new WBXMLException(sprintf("Loop count in while too high, code '%s' exceeded max. amount of permitted loops", $name));
         }
         return true;
     }
-    
+
     /**
      * Resets the inWhile counter.
      *
@@ -103,8 +84,8 @@ class WBXMLDecoder extends WBXMLDefs {
      *
      * @access public
      */
-    public function WBXMLDecoder($input) {
-        $this->log = defined('WBXML_DEBUG') && WBXML_DEBUG;
+    public function __construct($input) {
+        $this->log = ZLog::IsWbxmlDebugEnabled();
 
         $this->in = $input;
 
@@ -184,6 +165,9 @@ class WBXMLDecoder extends WBXMLDefs {
     public function getElementStartTag($tag) {
         $element = $this->getToken();
 
+        if (!$element)
+            return false;
+
         if($element[EN_TYPE] == EN_TYPE_STARTTAG && $element[EN_TAG] == $tag)
             return $element;
         else {
@@ -304,7 +288,7 @@ class WBXMLDecoder extends WBXMLDefs {
         }
 
         $el = $this->_getToken();
-        if($this->log)
+        if($this->log && $el)
             $this->logToken($el);
 
         return $el;
@@ -335,7 +319,18 @@ class WBXMLDecoder extends WBXMLDefs {
                 ZLog::Write(LOGLEVEL_WBXML,"I " . $spaces . "</" . $tag . ">");
                 break;
             case EN_TYPE_CONTENT:
-                ZLog::Write(LOGLEVEL_WBXML,"I " . $spaces . " " . $el[EN_CONTENT]);
+                // as we concatenate the string here, the entire content is copied.
+                // when sending an email with an attachment this single log line (which is never logged in INFO)
+                // requires easily additional 20 MB of RAM. See https://jira.z-hub.io/browse/ZP-1159
+                $messagesize = strlen($el[EN_CONTENT]);
+                if ($messagesize > 10240 && !defined('WBXML_DEBUGGING')) {
+                    $content = substr($el[EN_CONTENT], 0, 10240) . sprintf(" <log message with %d bytes truncated>", $messagesize);
+                }
+                else {
+                    $content = $el[EN_CONTENT];
+                }
+                // Log but make sure it's not truncated again (will be slightly bigger than 10KB)
+                ZLog::Write(LOGLEVEL_WBXML,"I " . $spaces . " " . $content, false);
                 break;
         }
     }
@@ -413,6 +408,21 @@ class WBXMLDecoder extends WBXMLDefs {
      * @return string
      */
     private function getTermStr() {
+        if (defined('WBXML_DEBUGGING') && WBXML_DEBUGGING === true) {
+            $str = "";
+            while (1) {
+                $in = $this->getByte();
+                if ($in == 0) {
+                    break;
+                }
+                else {
+                    $str .= chr($in);
+                }
+            }
+
+            return $str;
+        }
+
         // there is no unlimited "length" for stream_get_line,
         // so we use a huge value for "length" param (1Gb)
         // (0 == PHP_SOCK_CHUNK_SIZE (8192))
