@@ -6,29 +6,11 @@
 *
 * Created   :   01.10.2007
 *
-* Copyright 2007 - 2012 Zarafa Deutschland GmbH
+* Copyright 2007 - 2016 Zarafa Deutschland GmbH
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3,
-* as published by the Free Software Foundation with the following additional
-* term according to sec. 7:
-*
-* According to sec. 7 of the GNU Affero General Public License, version 3,
-* the terms of the AGPL are supplemented with the following terms:
-*
-* "Zarafa" is a registered trademark of Zarafa B.V.
-* "Z-Push" is a registered trademark of Zarafa Deutschland GmbH
-* The licensing of the Program under the AGPL does not imply a trademark license.
-* Therefore any rights, title and interest in our trademarks remain entirely with us.
-*
-* However, if you propagate an unmodified version of the Program you are
-* allowed to use the term "Z-Push" to indicate that you distribute the Program.
-* Furthermore you may use our trademarks where it is necessary to indicate
-* the intended purpose of a product or service provided you use it in accordance
-* with honest practices in industrial or commercial matters.
-* If you want to propagate modified versions of the Program under the name "Z-Push",
-* you may only do so if you have a written permission by Zarafa Deutschland GmbH
-* (to acquire a permission please contact Zarafa at trademark@zarafa.com).
+* as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -41,14 +23,13 @@
 * Consult LICENSE file for details
 ************************************************/
 
-
 class WBXMLEncoder extends WBXMLDefs {
     private $_dtd;
     private $_out;
 
-    private $_tagcp;
-    private $_attrcp;
+    private $_tagcp = 0;
 
+    private $log = false;
     private $logStack = array();
 
     // We use a delayed output mechanism in which we only output a tag when it actually has something
@@ -62,15 +43,11 @@ class WBXMLEncoder extends WBXMLDefs {
     private $multipart; // the content is multipart
     private $bodyparts;
 
-    public function WBXMLEncoder($output, $multipart = false) {
-        // make sure WBXML_DEBUG is defined. It should be at this point
-        if (!defined('WBXML_DEBUG')) define('WBXML_DEBUG', false);
+    public function __construct($output, $multipart = false) {
+        $this->log = ZLog::IsWbxmlDebugEnabled();
 
         $this->_out = $output;
 
-        $this->_tagcp = 0;
-        $this->_attrcp = 0;
-
         // reverse-map the DTD
         foreach($this->dtd["namespaces"] as $nsid => $nsname) {
             $this->_dtd["namespaces"][$nsname] = $nsid;
@@ -124,7 +101,6 @@ class WBXMLEncoder extends WBXMLDefs {
 
         if(!$nocontent) {
             $stackelem['tag'] = $tag;
-            $stackelem['attributes'] = $attributes;
             $stackelem['nocontent'] = $nocontent;
             $stackelem['sent'] = false;
 
@@ -134,7 +110,7 @@ class WBXMLEncoder extends WBXMLDefs {
             // output of an empty tag, and we therefore output the stack here
         } else {
             $this->_outputStack();
-            $this->_startTag($tag, $attributes, $nocontent);
+            $this->_startTag($tag, $nocontent);
         }
     }
 
@@ -157,16 +133,18 @@ class WBXMLEncoder extends WBXMLDefs {
             if(count($this->_stack) == 0 && $this->multipart == true) {
                 $this->processMultipart();
             }
+            if(count($this->_stack) == 0)
+                $this->writeLog();
         }
     }
 
     /**
-     * Puts content on the output stack
+     * Puts content on the output stack.
      *
-     * @param $content
+     * @param string $content
      *
      * @access public
-     * @return string
+     * @return
      */
     public function content($content) {
         // We need to filter out any \0 chars because it's the string terminator in WBXML. We currently
@@ -179,6 +157,33 @@ class WBXMLEncoder extends WBXMLDefs {
         $this->_content($content);
     }
 
+    /**
+     * Puts content of a stream on the output stack AND closes it.
+     *
+     * @param resource $stream
+     * @param boolean $asBase64     if true, the data will be encoded as base64, default: false
+     * @param boolean $opaque       if true, output the opaque data, default: false
+     *
+     * @access public
+     * @return
+     */
+    public function contentStream($stream, $asBase64 = false, $opaque = false) {
+        // Do not append filters to opaque data as it might contain null char
+        if (!$asBase64 && !$opaque) {
+            stream_filter_register('replacenullchar', 'ReplaceNullcharFilter');
+            $rnc_filter = stream_filter_append($stream, 'replacenullchar');
+        }
+
+        $this->_outputStack();
+        $this->_contentStream($stream, $asBase64, $opaque);
+
+        if (!$asBase64 && !$opaque) {
+            stream_filter_remove($rnc_filter);
+        }
+
+        fclose($stream);
+    }
+
     /**
      * Gets the value of multipart
      *
@@ -198,6 +203,8 @@ class WBXMLEncoder extends WBXMLDefs {
      * @return void
      */
     public function addBodypartStream($bp) {
+        if (!is_resource($bp))
+            throw new WBXMLException("WBXMLEncoder->addBodypartStream(): trying to add a ".gettype($bp)." instead of a stream");
         if ($this->multipart)
             $this->bodyparts[] = $bp;
     }
@@ -225,7 +232,7 @@ class WBXMLEncoder extends WBXMLDefs {
     private function _outputStack() {
         for($i=0;$i<count($this->_stack);$i++) {
             if(!$this->_stack[$i]['sent']) {
-                $this->_startTag($this->_stack[$i]['tag'], $this->_stack[$i]['attributes'], $this->_stack[$i]['nocontent']);
+                $this->_startTag($this->_stack[$i]['tag'], $this->_stack[$i]['nocontent']);
                 $this->_stack[$i]['sent'] = true;
             }
         }
@@ -237,8 +244,9 @@ class WBXMLEncoder extends WBXMLDefs {
      * @access private
      * @return
      */
-    private function _startTag($tag, $attributes = false, $nocontent = false) {
-        $this->logStartTag($tag, $attributes, $nocontent);
+    private function _startTag($tag, $nocontent = false) {
+        if ($this->log)
+            $this->logStartTag($tag, $nocontent);
 
         $mapping = $this->getMapping($tag);
 
@@ -251,31 +259,63 @@ class WBXMLEncoder extends WBXMLDefs {
         }
 
         $code = $mapping["code"];
-        if(isset($attributes) && is_array($attributes) && count($attributes) > 0) {
-            $code |= 0x80;
-        }
 
         if(!isset($nocontent) || !$nocontent)
             $code |= 0x40;
 
         $this->outByte($code);
-
-        if($code & 0x80)
-            $this->outAttributes($attributes);
     }
 
     /**
-     * Outputs actual data
+     * Outputs actual data.
      *
      * @access private
+     * @param string $content
      * @return
      */
     private function _content($content) {
-        $this->logContent($content);
-        $this->outByte(WBXML_STR_I);
+        if ($this->log)
+            $this->logContent($content);
+        $this->outByte(self::WBXML_STR_I);
         $this->outTermStr($content);
     }
 
+    /**
+     * Outputs actual data coming from a stream, optionally encoded as base64.
+     *
+     * @access private
+     * @param resource $stream
+     * @param boolean  $asBase64
+     * @return
+     */
+    private function _contentStream($stream, $asBase64, $opaque) {
+        $stat = fstat($stream);
+        // write full stream, including the finalizing terminator to the output stream (stuff outTermStr() would do)
+        if ($opaque) {
+            $this->outByte(self::WBXML_OPAQUE);
+            $this->outMBUInt($stat['size']);
+        }
+        else {
+            $this->outByte(self::WBXML_STR_I);
+        }
+
+        if ($asBase64) {
+            $out_filter = stream_filter_append($this->_out, 'convert.base64-encode');
+        }
+        $written = stream_copy_to_stream($stream, $this->_out);
+        if ($asBase64) {
+            stream_filter_remove($out_filter);
+        }
+        if (!$opaque) {
+            fwrite($this->_out, chr(0));
+        }
+
+        if ($this->log) {
+            // data is out, do some logging
+            $this->logContent(sprintf("<<< written %d of %d bytes of %s data >>>", $written, $stat['size'], $asBase64 ? "base64 encoded":"plain"));
+        }
+    }
+
     /**
      * Outputs an actual end tag
      *
@@ -283,8 +323,9 @@ class WBXMLEncoder extends WBXMLDefs {
      * @return
      */
     private function _endTag() {
-        $this->logEndTag();
-        $this->outByte(WBXML_END);
+        if ($this->log)
+            $this->logEndTag();
+        $this->outByte(self::WBXML_END);
     }
 
     /**
@@ -300,24 +341,39 @@ class WBXMLEncoder extends WBXMLDefs {
     }
 
     /**
-     * Outputs a string table
+     * Output the multibyte integers to the stream.
      *
-     * @param $uint
+     * A multi-byte integer consists of a series of octets,
+     * where the most significant bit is the continuation flag
+     * and the remaining seven bits are a scalar value.
+     * The octets are arranged in a big-endian order,
+     * eg, the most significant seven bits are transmitted first.
+     *
+     * @see https://www.w3.org/1999/06/NOTE-wbxml-19990624/#_Toc443384895
+     *
+     * @param int $uint
      *
      * @access private
-     * @return
+     * @return void
      */
     private function outMBUInt($uint) {
-        while(1) {
+        if ($uint == 0x0) {
+            return $this->outByte($uint);
+        }
+
+        $out = '';
+
+        for ($i = 0; $uint != 0; $i++) {
             $byte = $uint & 0x7f;
             $uint = $uint >> 7;
-            if($uint == 0) {
-                $this->outByte($byte);
-                break;
-            } else {
-                $this->outByte($byte | 0x80);
+            if ($i == 0) {
+                $out = chr($byte) . $out;
+            }
+            else {
+                $out = chr($byte | 0x80) . $out;
             }
         }
+        fwrite($this->_out, $out);
     }
 
     /**
@@ -333,20 +389,6 @@ class WBXMLEncoder extends WBXMLDefs {
         fwrite($this->_out, chr(0));
     }
 
-    /**
-     * Output attributes
-     * We don't actually support this, because to do so, we would have
-     * to build a string table before sending the data (but we can't
-     * because we're streaming), so we'll just send an END, which just
-     * terminates the attribute list with 0 attributes.
-     *
-     * @access private
-     * @return
-     */
-    private function outAttributes() {
-        $this->outByte(WBXML_END);
-    }
-
     /**
      * Switches the codepage
      *
@@ -356,7 +398,7 @@ class WBXMLEncoder extends WBXMLDefs {
      * @return
      */
     private function outSwitchPage($page) {
-        $this->outByte(WBXML_SWITCH_PAGE);
+        $this->outByte(self::WBXML_SWITCH_PAGE);
         $this->outByte($page);
     }
 
@@ -420,16 +462,12 @@ class WBXMLEncoder extends WBXMLDefs {
      * Logs a StartTag to ZLog
      *
      * @param $tag
-     * @param $attr
      * @param $nocontent
      *
      * @access private
      * @return
      */
-    private function logStartTag($tag, $attr, $nocontent) {
-        if(!WBXML_DEBUG)
-            return;
-
+    private function logStartTag($tag, $nocontent) {
         $spaces = str_repeat(" ", count($this->logStack));
         if($nocontent)
             ZLog::Write(LOGLEVEL_WBXML,"O " . $spaces . " <$tag/>");
@@ -446,9 +484,6 @@ class WBXMLEncoder extends WBXMLDefs {
      * @return
      */
     private function logEndTag() {
-        if(!WBXML_DEBUG)
-            return;
-
         $spaces = str_repeat(" ", count($this->logStack));
         $tag = array_pop($this->logStack);
         ZLog::Write(LOGLEVEL_WBXML,"O " . $spaces . "</$tag>");
@@ -457,15 +492,12 @@ class WBXMLEncoder extends WBXMLDefs {
     /**
      * Logs content to ZLog
      *
-     * @param $content
+     * @param string $content
      *
      * @access private
      * @return
      */
     private function logContent($content) {
-        if(!WBXML_DEBUG)
-            return;
-
         $spaces = str_repeat(" ", count($this->logStack));
         ZLog::Write(LOGLEVEL_WBXML,"O " . $spaces . $content);
     }
@@ -483,25 +515,40 @@ class WBXMLEncoder extends WBXMLDefs {
         $nrBodyparts = $this->getBodypartsCount();
         $blockstart = (($nrBodyparts + 1) * 2) * 4 + 4;
 
-        $data = pack("iii", ($nrBodyparts + 1), $blockstart, $len);
+        fwrite($this->_out, pack("iii", ($nrBodyparts + 1), $blockstart, $len));
 
-        ob_start(null, 1048576);
-
-        foreach ($this->bodyparts as $bp) {
+        foreach ($this->bodyparts as $i=>$bp) {
             $blockstart = $blockstart + $len;
             $len = fstat($bp);
             $len = (isset($len['size'])) ? $len['size'] : 0;
-            $data .= pack("ii", $blockstart, $len);
+            if ($len == 0) {
+                ZLog::Write(LOGLEVEL_WARN, sprintf("WBXMLEncoder->processMultipart(): the length of the body part at position %d is 0", $i));
+            }
+            fwrite($this->_out, pack("ii", $blockstart, $len));
         }
 
-        fwrite($this->_out, $data);
         fwrite($this->_out, $buffer);
+
         foreach($this->bodyparts as $bp) {
-            while (!feof($bp)) {
-                fwrite($this->_out, fread($bp, 4096));
-            }
+            stream_copy_to_stream($bp, $this->_out);
+            fclose($bp);
         }
     }
-}
 
-?>
\ No newline at end of file
+    /**
+     * Writes the sent WBXML data to the log if it is not bigger than 512K.
+     *
+     * @access private
+     * @return void
+     */
+    private function writeLog() {
+        if (ob_get_length() === false) {
+            $data = "output buffer disabled";
+        } elseif (ob_get_length() < 524288) {
+            $data = base64_encode(ob_get_contents());
+        } else {
+            $data = "more than 512K of data";
+        }
+        ZLog::Write(LOGLEVEL_WBXML, "WBXML-OUT: ". $data, false);
+    }
+}
