@@ -11,25 +11,7 @@
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3,
-* as published by the Free Software Foundation with the following additional
-* term according to sec. 7:
-*
-* According to sec. 7 of the GNU Affero General Public License, version 3,
-* the terms of the AGPL are supplemented with the following terms:
-*
-* "Zarafa" is a registered trademark of Zarafa B.V.
-* "Z-Push" is a registered trademark of Zarafa Deutschland GmbH
-* The licensing of the Program under the AGPL does not imply a trademark license.
-* Therefore any rights, title and interest in our trademarks remain entirely with us.
-*
-* However, if you propagate an unmodified version of the Program you are
-* allowed to use the term "Z-Push" to indicate that you distribute the Program.
-* Furthermore you may use our trademarks where it is necessary to indicate
-* the intended purpose of a product or service provided you use it in accordance
-* with honest practices in industrial or commercial matters.
-* If you want to propagate modified versions of the Program under the name "Z-Push",
-* you may only do so if you have a written permission by Zarafa Deutschland GmbH
-* (to acquire a permission please contact Zarafa at trademark@zarafa.com).
+* as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -55,11 +37,14 @@ class Webservice {
      * @throws SoapFault
      */
     public function Handle($commandCode) {
-        if (Request::GetDeviceType() !== "webservice" || Request::GetDeviceID() !== "webservice")
+        if (Request::GetDeviceType() !== "webservice" || Request::GetDeviceID() !== "webservice") {
             throw new FatalException("Invalid device id and type for webservice execution");
+        }
 
-        if (Request::GetGETUser() != Request::GetAuthUser())
-            ZLog::Write(LOGLEVEL_INFO, sprintf("Webservice::HandleWebservice('%s'): user '%s' executing action for user '%s'", $commandCode, Request::GetAuthUser(), Request::GetGETUser()));
+        $user = (Request::GetImpersonatedUser()) ? Request::GetImpersonatedUser() : Request::GetGETUser();
+        if ($user != Request::GetAuthUser()) {
+            ZLog::Write(LOGLEVEL_INFO, sprintf("Webservice::HandleWebservice('%s'): user '%s' executing action for user '%s'", $commandCode, Request::GetAuthUser(), $user));
+        }
 
         // initialize non-wsdl soap server
         $this->server = new SoapServer(null, array('uri' => "http://z-push.org/webservice"));
@@ -67,24 +52,27 @@ class Webservice {
         // the webservice command is handled by its class
         if ($commandCode == ZPush::COMMAND_WEBSERVICE_DEVICE) {
             // check if the authUser has admin permissions to get data on the GETUser's device
-            if(ZPush::GetBackend()->Setup(Request::GetGETUser(), true) == false)
-                throw new AuthenticationRequiredException(sprintf("Not enough privileges of '%s' to setup for user '%s': Permission denied", Request::GetAuthUser(), Request::GetGETUser()));
+            if (ZPush::GetBackend()->Setup($user, true) == false) {
+                throw new AuthenticationRequiredException(sprintf("Not enough privileges of '%s' to setup for user '%s': Permission denied", Request::GetAuthUser(), $user));
+            }
 
             ZLog::Write(LOGLEVEL_DEBUG, sprintf("Webservice::HandleWebservice('%s'): executing WebserviceDevice service", $commandCode));
             $this->server->setClass("WebserviceDevice");
         }
-        else if ($commandCode == ZPush::COMMAND_WEBSERVICE_INFO) {
+        elseif ($commandCode == ZPush::COMMAND_WEBSERVICE_INFO) {
             ZLog::Write(LOGLEVEL_DEBUG, sprintf("Webservice::HandleWebservice('%s'): executing WebserviceInfo service", $commandCode));
             $this->server->setClass("WebserviceInfo");
         }
-        else if ($commandCode == ZPush::COMMAND_WEBSERVICE_USERS) {
-            if (!defined("ALLOW_WEBSERVICE_USERS_ACCESS") || ALLOW_WEBSERVICE_USERS_ACCESS !== true)
+        elseif ($commandCode == ZPush::COMMAND_WEBSERVICE_USERS) {
+            if (!defined("ALLOW_WEBSERVICE_USERS_ACCESS") || ALLOW_WEBSERVICE_USERS_ACCESS !== true) {
                 throw new HTTPReturnCodeException("Access to the WebserviceUsers service is disabled in configuration. Enable setting ALLOW_WEBSERVICE_USERS_ACCESS", 403);
+            }
 
             ZLog::Write(LOGLEVEL_DEBUG, sprintf("Webservice::HandleWebservice('%s'): executing WebserviceUsers service", $commandCode));
 
-            if(ZPush::GetBackend()->Setup("SYSTEM", true) == false)
+            if (ZPush::GetBackend()->Setup("SYSTEM", true) == false) {
                 throw new AuthenticationRequiredException(sprintf("User '%s' has no admin privileges", Request::GetAuthUser()));
+            }
 
             $this->server->setClass("WebserviceUsers");
         }
