@@ -28,13 +28,13 @@
 class WebserviceDevice {
 
     /**
-     * Returns a list of all known devices of the Request::GetGETUser().
+     * Returns a list of all known devices of the requested user.
      *
      * @access public
      * @return array
      */
     public function ListDevicesDetails() {
-        $user = Request::GetGETUser();
+        $user = Request::GetImpersonatedUser() ? Request::GetImpersonatedUser() : Request::GetGETUser();
         $devices = ZPushAdmin::ListDevices($user);
         $output = array();
 
@@ -48,7 +48,7 @@ class WebserviceDevice {
     }
 
     /**
-     * Returns the details of a given deviceid of the Request::GetGETUser().
+     * Returns the details of a given deviceid of the requested user.
      *
      * @param boolean   $withHierarchyCache (opt) includes the HierarchyCache - default: false
      *
@@ -56,7 +56,7 @@ class WebserviceDevice {
      * @return ASDevice object
      */
     public function GetDeviceDetails($deviceId, $withHierarchyCache = false) {
-        $user = Request::GetGETUser();
+        $user = Request::GetImpersonatedUser() ? Request::GetImpersonatedUser() : Request::GetGETUser();
         $deviceId = preg_replace("/[^A-Za-z0-9]/", "", $deviceId);
         ZLog::Write(LOGLEVEL_INFO, sprintf("WebserviceDevice::GetDeviceDetails('%s'): getting device details from state of user '%s'", $deviceId, $user));
 
@@ -65,7 +65,7 @@ class WebserviceDevice {
     }
 
     /**
-     * Remove all state data for a device of the Request::GetGETUser().
+     * Remove all state data for a device of the requested user.
      *
      * @param string    $deviceId       the device id
      *
@@ -74,10 +74,11 @@ class WebserviceDevice {
      * @throws SoapFault
      */
     public function RemoveDevice($deviceId) {
+        $user = Request::GetImpersonatedUser() ? Request::GetImpersonatedUser() : Request::GetGETUser();
         $deviceId = preg_replace("/[^A-Za-z0-9]/", "", $deviceId);
-        ZLog::Write(LOGLEVEL_INFO, sprintf("WebserviceDevice::RemoveDevice('%s'): remove device state data of user '%s'", $deviceId, Request::GetGETUser()));
+        ZLog::Write(LOGLEVEL_INFO, sprintf("WebserviceDevice::RemoveDevice('%s'): remove device state data of user '%s'", $deviceId, $user));
 
-        if (! ZPushAdmin::RemoveDevice(Request::GetGETUser(), $deviceId)) {
+        if (! ZPushAdmin::RemoveDevice($user, $deviceId)) {
             ZPush::GetTopCollector()->AnnounceInformation(ZLog::GetLastMessage(LOGLEVEL_ERROR), true);
             throw new SoapFault("ERROR", ZLog::GetLastMessage(LOGLEVEL_ERROR));
         }
@@ -87,7 +88,7 @@ class WebserviceDevice {
     }
 
     /**
-     * Marks a device of the Request::GetGETUser() to be remotely wiped.
+     * Marks a device of the requested user to be remotely wiped.
      *
      * @param string    $deviceId       the device id
      *
@@ -96,10 +97,14 @@ class WebserviceDevice {
      * @throws SoapFault
      */
     public function WipeDevice($deviceId) {
+        if (Request::GetImpersonatedUser()) {
+            throw new SoapFault("ERROR", "Impersonated user is not allowed to wipe devices.");
+        }
         $deviceId = preg_replace("/[^A-Za-z0-9]/", "", $deviceId);
-        ZLog::Write(LOGLEVEL_INFO, sprintf("WebserviceDevice::WipeDevice('%s'): mark device of user '%s' for remote wipe", $deviceId, Request::GetGETUser()));
+        $user = Request::GetGETUser();
+        ZLog::Write(LOGLEVEL_INFO, sprintf("WebserviceDevice::WipeDevice('%s'): mark device of user '%s' for remote wipe", $deviceId, $user));
 
-        if (! ZPushAdmin::WipeDevice(Request::GetAuthUser(), Request::GetGETUser(), $deviceId)) {
+        if (! ZPushAdmin::WipeDevice(Request::GetAuthUser(), $user, $deviceId)) {
             ZPush::GetTopCollector()->AnnounceInformation(ZLog::GetLastMessage(LOGLEVEL_ERROR), true);
             throw new SoapFault("ERROR", ZLog::GetLastMessage(LOGLEVEL_ERROR));
         }
@@ -109,7 +114,7 @@ class WebserviceDevice {
     }
 
     /**
-     * Sets device options of the Request::GetGETUser().
+     * Sets device options of the requested user.
      *
      * @param string    $deviceId       the device id
      * @param int       $filtertype     SYNC_FILTERTYPE_1DAY to SYNC_FILTERTYPE_ALL, false to ignore
@@ -120,9 +125,10 @@ class WebserviceDevice {
      */
     public function SetDeviceOptions($deviceId, $filtertype) {
         $deviceId = preg_replace("/[^A-Za-z0-9]/", "", $deviceId);
-        ZLog::Write(LOGLEVEL_INFO, sprintf("WebserviceDevice::SetDeviceOptions('%s', '%s'): set FilterType to '%s'", $deviceId, Request::GetGETUser(), Utils::PrintAsString($filtertype)));
+        $user = Request::GetImpersonatedUser() ? Request::GetImpersonatedUser() : Request::GetGETUser();
+        ZLog::Write(LOGLEVEL_INFO, sprintf("WebserviceDevice::SetDeviceOptions('%s', '%s'): set FilterType to '%s'", $deviceId, $user, Utils::PrintAsString($filtertype)));
 
-        if (! ZPushAdmin::SetDeviceOptions(Request::GetGETUser(), $deviceId, $filtertype)) {
+        if (! ZPushAdmin::SetDeviceOptions($user, $deviceId, $filtertype)) {
             ZPush::GetTopCollector()->AnnounceInformation(ZLog::GetLastMessage(LOGLEVEL_ERROR), true);
             throw new SoapFault("ERROR", ZLog::GetLastMessage(LOGLEVEL_ERROR));
         }
@@ -132,7 +138,7 @@ class WebserviceDevice {
     }
 
     /**
-     * Marks a device of the Request::GetGETUser() for resynchronization.
+     * Marks a device of the requested user for resynchronization.
      *
      * @param string    $deviceId       the device id
      *
@@ -142,9 +148,10 @@ class WebserviceDevice {
      */
     public function ResyncDevice($deviceId) {
         $deviceId = preg_replace("/[^A-Za-z0-9]/", "", $deviceId);
-        ZLog::Write(LOGLEVEL_INFO, sprintf("WebserviceDevice::ResyncDevice('%s'): mark device of user '%s' for resynchronization", $deviceId, Request::GetGETUser()));
+        $user = Request::GetImpersonatedUser() ? Request::GetImpersonatedUser() : Request::GetGETUser();
+        ZLog::Write(LOGLEVEL_INFO, sprintf("WebserviceDevice::ResyncDevice('%s'): mark device of user '%s' for resynchronization", $deviceId, $user));
 
-        if (! ZPushAdmin::ResyncDevice(Request::GetGETUser(), $deviceId)) {
+        if (! ZPushAdmin::ResyncDevice($user, $deviceId)) {
             ZPush::GetTopCollector()->AnnounceInformation(ZLog::GetLastMessage(LOGLEVEL_ERROR), true);
             throw new SoapFault("ERROR", ZLog::GetLastMessage(LOGLEVEL_ERROR));
         }
@@ -154,7 +161,7 @@ class WebserviceDevice {
     }
 
     /**
-     * Marks a folder of a device of the Request::GetGETUser() for resynchronization.
+     * Marks a folder of a device of the requested user for resynchronization.
      *
      * @param string    $deviceId       the device id
      * @param string    $folderId       the folder id
@@ -166,9 +173,10 @@ class WebserviceDevice {
     public function ResyncFolder($deviceId, $folderId) {
         $deviceId = preg_replace("/[^A-Za-z0-9]/", "", $deviceId);
         $folderId = preg_replace("/[^A-Za-z0-9]/", "", $folderId);
-        ZLog::Write(LOGLEVEL_INFO, sprintf("WebserviceDevice::ResyncFolder('%s','%s'): mark folder of a device of user '%s' for resynchronization", $deviceId, $folderId, Request::GetGETUser()));
+        $user = Request::GetImpersonatedUser() ? Request::GetImpersonatedUser() : Request::GetGETUser();
+        ZLog::Write(LOGLEVEL_INFO, sprintf("WebserviceDevice::ResyncFolder('%s','%s'): mark folder of a device of user '%s' for resynchronization", $deviceId, $folderId, $user));
 
-        if (! ZPushAdmin::ResyncFolder(Request::GetGETUser(), $deviceId, $folderId)) {
+        if (! ZPushAdmin::ResyncFolder($user, $deviceId, $folderId)) {
             ZPush::GetTopCollector()->AnnounceInformation(ZLog::GetLastMessage(LOGLEVEL_ERROR), true);
             throw new SoapFault("ERROR", ZLog::GetLastMessage(LOGLEVEL_ERROR));
         }
@@ -178,7 +186,7 @@ class WebserviceDevice {
     }
 
     /**
-     * Returns a list of all additional folders of the given device and the Request::GetGETUser().
+     * Returns a list of all additional folders of the given device and the requested user.
      *
      * @param string    $deviceId       device id that should be listed.
      *
@@ -186,7 +194,7 @@ class WebserviceDevice {
      * @return array
      */
     public function AdditionalFolderList($deviceId) {
-        $user = Request::GetGETUser();
+        $user = Request::GetImpersonatedUser() ? Request::GetImpersonatedUser() : Request::GetGETUser();
         $deviceId = preg_replace("/[^A-Za-z0-9]/", "", $deviceId);
         $folders = ZPushAdmin::AdditionalFolderList($user, $deviceId);
         if ($folders === false) {
@@ -215,7 +223,7 @@ class WebserviceDevice {
     }
 
     /**
-     * Adds an additional folder to the given device and the Request::GetGETUser().
+     * Adds an additional folder to the given device and the requested user.
      *
      * @param string    $deviceId       device id the folder should be added to.
      * @param string    $add_store      the store where this folder is located, e.g. "SYSTEM" (for public folder) or an username/email address.
@@ -228,7 +236,7 @@ class WebserviceDevice {
      * @return boolean
      */
     public function AdditionalFolderAdd($deviceId, $add_store, $add_folderid, $add_name, $add_type, $add_flags) {
-        $user = Request::GetGETUser();
+        $user = Request::GetImpersonatedUser() ? Request::GetImpersonatedUser() : Request::GetGETUser();
         $deviceId = preg_replace("/[^A-Za-z0-9]/", "", $deviceId);
         $add_folderid = preg_replace("/[^A-Za-z0-9]/", "", $add_folderid);
         $add_type = preg_replace("/[^0-9]/", "", $add_type);
@@ -246,7 +254,7 @@ class WebserviceDevice {
     }
 
     /**
-     * Updates the name of an additional folder to the given device and the Request::GetGETUser().
+     * Updates the name of an additional folder to the given device and the requested user.
      *
      * @param string    $deviceId       device id of where the folder should be updated.
      * @param string    $add_folderid   the folder id of the additional folder.
@@ -257,7 +265,7 @@ class WebserviceDevice {
      * @return boolean
      */
     public function AdditionalFolderEdit($deviceId, $add_folderid, $add_name, $add_flags) {
-        $user = Request::GetGETUser();
+        $user = Request::GetImpersonatedUser() ? Request::GetImpersonatedUser() : Request::GetGETUser();
         $deviceId = preg_replace("/[^A-Za-z0-9]/", "", $deviceId);
         $add_folderid = preg_replace("/[^A-Za-z0-9]/", "", $add_folderid);
         $add_flags = preg_replace("/[^0-9]/", "", $add_flags);
@@ -274,7 +282,7 @@ class WebserviceDevice {
     }
 
     /**
-     * Removes an additional folder from the given device and the Request::GetGETUser().
+     * Removes an additional folder from the given device and the requested user.
      *
      * @param string    $deviceId       device id of where the folder should be removed.
      * @param string    $add_folderid   the folder id of the additional folder.
@@ -283,7 +291,7 @@ class WebserviceDevice {
      * @return boolean
      */
     public function AdditionalFolderRemove($deviceId, $add_folderid) {
-        $user = Request::GetGETUser();
+        $user = Request::GetImpersonatedUser() ? Request::GetImpersonatedUser() : Request::GetGETUser();
         $deviceId = preg_replace("/[^A-Za-z0-9]/", "", $deviceId);
         $add_folderid = preg_replace("/[^A-Za-z0-9]/", "", $add_folderid);
 
@@ -299,7 +307,7 @@ class WebserviceDevice {
     }
 
     /**
-     * Sets a list of additional folders of one store to the given device and the Request::GetGETUser().
+     * Sets a list of additional folders of one store to the given device and the requested user.
      * If there are additional folders for this store, that are not in the list they will be removed.
      *
      * @param string    $deviceId       device id the folder should be added to.
@@ -316,7 +324,7 @@ class WebserviceDevice {
      * @return boolean
      */
     public function AdditionalFolderSetList($deviceId, $set_store, $set_folders) {
-        $user = Request::GetGETUser();
+        $user = Request::GetImpersonatedUser() ? Request::GetImpersonatedUser() : Request::GetGETUser();
         $deviceId = preg_replace("/[^A-Za-z0-9]/", "", $deviceId);
         array_walk($set_folders, function(&$folder) {
             if (!isset($folder['folderid']))    $folder['folderid'] = "";
