@@ -23,4 +23,14 @@
 * Consult LICENSE file for details
 ************************************************/
 
-define("ZPUSH_VERSION", "2.3.8");
\ No newline at end of file
+if (!defined("ZPUSH_VERSION")) {
+    $path = escapeshellarg(dirname(realpath($_SERVER['SCRIPT_FILENAME'])));
+    $branch = trim(exec("hash git 2>/dev/null && cd $path >/dev/null 2>&1 && git branch --no-color 2>/dev/null | sed -e '/^[^*]/d' -e \"s/* \(.*\)/\\1/\""));
+    $version = exec("hash git 2>/dev/null && cd $path >/dev/null 2>&1 && git describe  --always 2>/dev/null");
+    if ($branch && $version) {
+        define("ZPUSH_VERSION", $branch .'-'. $version);
+    }
+    else {
+        define("ZPUSH_VERSION", "GIT");
+    }
+}
\ No newline at end of file
